#!/usr/bin/env python
import argparse
import os.path
import grpc
import base64
import hashlib
import SurfStoreBasic_pb2
import SurfStoreBasic_pb2_grpc

from config_reader import SurfStoreConfigReader

def sha256(s):
    m = hashlib.sha256()
    m.update(str.encode(s))
    return base64.b64encode(m.digest())

def parse_args():
    parser = argparse.ArgumentParser(description="SurfStore client")
    parser.add_argument("config_file", type=str,
                        help="Path to configuration file")
    return parser.parse_args()


def get_metadata_stub(config):
    channel = grpc.insecure_channel('127.0.0.1:%d' % config.metadata_ports[1])
    stub = SurfStoreBasic_pb2_grpc.MetadataStoreStub(channel)
    return stub


def get_block_stub(config):
    channel = grpc.insecure_channel('127.0.0.1:%d' % config.block_port)
    stub = SurfStoreBasic_pb2_grpc.BlockStoreStub(channel)
    return stub

def ensure(boolean_expression):
    if boolean_expression == False:
        print ("Assertion Failed")
        raise SystemExit

def stringToBlock(string):
    builder = SurfStoreBasic_pb2.Block()
    try:
        #builder.data = ByteString.copyfrom(string,"UTF-8")
        builder.data = string.encode()
    except:
        print ("Encoding issues")
        raise SystemExit

    builder.hash = sha256(string)

    return builder
    
def run(config):
    metadata_stub = get_metadata_stub(config)
    block_stub = get_block_stub(config)

    #metadata_stub.Ping(SurfStoreBasic_pb2.Empty())
    #print("Successfully pinged the Metadata server")

    block_stub.Ping(SurfStoreBasic_pb2.Empty())
    print("Successfully pinged the Blockstore server")
 
    # Implement your client here
    b1 = stringToBlock("block_013984r9438fhwe^#&*(#@^)(")
    b2 = stringToBlock("block_02")
    print ("Check for blocks using HasBlock") 
    ensure(block_stub.HasBlock(b1).answer == False)
    ensure(block_stub.HasBlock(b2).answer == False)
    
    print ("Storing block b1") 
    block_stub.StoreBlock(b1)
    print ("ensuring block b1 is stored") 
    ensure(block_stub.HasBlock(b1).answer == True)
    
    print ("Storing block b2") 
    block_stub.StoreBlock(b2)
    print ("ensuring block b2 is stored") 
    ensure(block_stub.HasBlock(b2).answer == True)

    print ("Getting block b1")
    b1prime = block_stub.GetBlock(b1)
    print ("ensuring remote block b1 is same as local block b1") 
    ensure(b1prime.hash == b1.hash)
    ensure(b1prime.data == b1.data)

    print ("Passed all the tests.. yay !!!")

if __name__ == "__main__":
    args = parse_args()
    config = SurfStoreConfigReader(args.config_file)

    run(config)
